//tema_main.c

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <pthread.h>
#include <sys/time.h>
#include "mrtu.h"
#include "main.h" 		//подключаем наш заголовочный файл 
#include "rsocket.h"		//подключаем заголовочный файл для работы с сокетами

int	ipport;	//наш порт прослушивания

int	num_work;	//сётчик для работы
U16	time_mark;	//маркер времени
/*
************************
массив рабочих регистров
************************
*/
I16	regs[_cnt_regs];
I16	wmotors[4];
I16	wservos[18];
I16	rmotors[4];
I16	rservos[18];

int main(int argc, char **argv)
{
	int scnt 		= 1; //счётчик для анализа входных параметров
	int res 		= 1;
	int cnt;
	int baudrate;
	int symbols;
	int symwait;
	int wait_dev		= 50000;
	char	serialname[255] = {0};
	//без параметров
	if(argc < 2)
	{
		printf("\nParametrs not defined!\r\n");
		printf("\"--help\" for information\n");
		return 1;
	}
	//
	while(scnt < argc)
	{
		//подсказка
		if(strcmp(argv[scnt], "--help") == 0)
		{
			printf(__help);
			return 0;
		}
		//порт TCP
                else if(strcmp(argv[scnt], "-p") == 0)
                {
                        scnt++;
                        if(!(scnt < argc))
                                return 2;
                        ipport = atoi(argv[scnt]);
                        if(ipport == 0)
				return 3;
		}
		//последовательный порт
		else if(strcmp(argv[scnt], "-sp") == 0)
                {
                        scnt++;
                        if(!(scnt < argc))
                                return 2;
			//порт
                        strcpy(&serialname[0], argv[scnt]);
			scnt++;
                        if(!(scnt < argc))
                                return 2;
			//скорость
			baudrate = atoi(argv[scnt]);
			scnt++;
                        if(!(scnt < argc))
                                return 2;
			//ожидание в символах
			symbols = atoi(argv[scnt]);
			symwait = (int)((1000000.0f / (float)(baudrate)) * 10 * (float)(symbols));
		}
		//ожидание цикла устройства
		else if(strcmp(argv[scnt], "-wd") == 0)
                {
                        scnt++;
                        if(!(scnt < argc))
                                return 2;
			wait_dev = atoi(argv[scnt]) * 1000;
		}
		else
			return 4;
		scnt++;
	}
	//открытие прослушивающего сокета - запуск сервера
	res = SocketStart();
	printf("---- opening tcp/ip  %u ----\r\n", ipport);
	if(res != 0)
	{
		printf("ERROR starting server\n");
		return 5;
	}
	//соединяемся с Arduino
	res = MRTU_Connect(&serialname[0], baudrate, symwait);
	if(res != 0)
	{
		SocketStop();
		printf("ERROR connecting to Arduino\n");
		return 5;
	}
	//обнуляем рабочий счётчик
	num_work 	= 0;
	//обнуляем счётчик времени 
	time_mark	= 0;
	//моторы
	cnt = 0;
	while(cnt < 4)
	{
		wmotors[cnt] = 0;
		rmotors[cnt] = 0;
		cnt++;
	}
	//сервомашины
	cnt = 0;
	while(cnt < 18)
	{
		wservos[cnt] = 0;
		rservos[cnt] = 0;
		cnt++;
	}
	//наш цикл Main
	while(1)
	{
		MainWork();	//Рабочая функция
		usleep(wait_dev);	//ожтдание (uS)
	}
	//
	return 0;
}

//рабочий модуль
void	MainWork(void)
{
	int	count;
	int	addr;
	int	cnt;
	I16	res;	//результат выполнения функции
	//
	pthread_mutex_lock(&ServerMutex);
	//копирование заданий регистров
	addr 	= 0;
	count	= 0;
	while(count < _cnt_regs)
	{
		regs[count] = (*(I16*)(&wbuffrom[addr]));
		addr += 2;	
		count++;
	}
	switch(regs[_mode_contrl])
	{
		//приращение на единицу
		case _mode_ctrl1:
			//моторы
			cnt = 0;
			while(cnt < 4)
			{
				if((wmotors[cnt] < regs[_num_vol_m1 + cnt]) &&
				(wmotors[cnt] < 255))
					wmotors[cnt]++;
				else if((wmotors[cnt] > regs[_num_vol_m1 + cnt]) &&
				(wmotors[cnt] > -255))
					wmotors[cnt]--;
				cnt++;
			}
			//сервомашины
			cnt = 0;
			while(cnt < 18)
			{
				if((wservos[cnt] < regs[_num_pos_s1 + cnt]) &&
				(wservos[cnt] < 180))
					wservos[cnt]++;
				else if((wservos[cnt] > regs[_num_pos_s1 + cnt]) &&
				(wservos[cnt] > -180))
					wservos[cnt]--;
				cnt++;
			}
			break;
		//обычная установка
		default:
			//копирование данных для реализации
			memcpy(&wmotors[0], &regs[_num_vol_m1], 8);	//моторы
			memcpy(&wservos[0], &regs[_num_pos_s1], 36);	//сервомашины
			//test
			//memcpy(&rmotors[0], &wmotors[0], 8);
			//memcpy(&rservos[0], &wservos[0], 36);
			//копирование данных для отчёта
			memcpy(&regs[_rnum_vol_m1], &rmotors[0], 8);	//моторы
			memcpy(&regs[_rnum_pos_s1], &rservos[0], 36);	//сервомашины
			break;
	}
	//маркер времени
	regs[_time_mark] = time_mark;
	//копировние значений регистров для отчёта
	addr 	= 0;
	count	= 0;
	while(count < _cnt_regs)
	{
		(*(I16*)(&wbufto[addr])) = regs[count];
		addr += 2;	
		count++;
	}
	pthread_mutex_unlock(&ServerMutex);
	//
	switch(num_work)
	{
		case 0:	//задание для 4-х моторов
			res = MRTU_CMD_16(1, _num_vol_m1, (U16*)(&wmotors[0]), 4);
			num_work++;
			break;
		case 1:	//чтение значения 4-х моторов
			res = MRTU_CMD_3(1, _num_vol_m1, (U16*)(&rmotors[0]), 4);
			num_work++;
			break;
		case 2:	//управление положением камеры - два сервопривода
			res = MRTU_CMD_16(1, _num_pos_s1, (U16*)(&wservos[0]), 2);
			num_work++;
			break;
		case 3:	//чтение положения камеры - два сервопривода
			res = MRTU_CMD_3(1, _num_pos_s1, (U16*)(&rservos[0]), 2);
			num_work = 0;
			break;
		default:
			num_work = 0;
			break;
	}
	//изменяем маркер времени
	if(time_mark < 0xFFFF)
		time_mark++;
	else
		time_mark = 0;
}
